@@ -6,20 +6,16 @@
 # rather than the one from site-packages.
 
 import os
+
 import pytest
 
 
 def pytest_runtest_setup(item):
     fname = item.fspath.strpath
-    if fname.endswith(os.path.join("keras", "_generator.py")) or fname.endswith(
-        "miscellaneous.rst"
-    ):
-        try:
-            import keras  # noqa
-        except ImportError:
-            pytest.skip("The keras package is not installed.")
-    elif fname.endswith(os.path.join("tensorflow", "_generator.py")) or fname.endswith(
-        "miscellaneous.rst"
+    if (
+        fname.endswith(os.path.join("keras", "_generator.py"))
+        or fname.endswith(os.path.join("tensorflow", "_generator.py"))
+        or fname.endswith("miscellaneous.rst")
     ):
         try:
             import tensorflow  # noqa
