@@ -5,19 +5,27 @@
 # License: MIT
 
 import copy
-import inspect
 import numbers
 import warnings
 
 import numpy as np
-from joblib import Parallel
+import sklearn
 from sklearn.base import clone
 from sklearn.ensemble import AdaBoostClassifier, BaggingClassifier
 from sklearn.ensemble._bagging import _parallel_decision_function
 from sklearn.ensemble._base import _partition_estimators
-from sklearn.utils.fixes import delayed
+from sklearn.exceptions import NotFittedError
+from sklearn.utils import parse_version
+from sklearn.utils._tags import _safe_tags
 from sklearn.utils.validation import check_is_fitted
 
+try:
+    # scikit-learn >= 1.2
+    from sklearn.utils.parallel import Parallel, delayed
+except (ImportError, ModuleNotFoundError):
+    from joblib import Parallel
+    from sklearn.utils.fixes import delayed
+
 from ..base import _ParamsValidationMixin
 from ..pipeline import Pipeline
 from ..under_sampling import RandomUnderSampler
@@ -26,9 +34,11 @@ from ..utils import Substitution, check_sampling_strategy, check_target_type
 from ..utils._available_if import available_if
 from ..utils._docstring import _n_jobs_docstring, _random_state_docstring
 from ..utils._param_validation import Interval, StrOptions
+from ..utils.fixes import _fit_context
 from ._common import _bagging_parameter_constraints, _estimator_has
 
 MAX_INT = np.iinfo(np.int32).max
+sklearn_version = parse_version(sklearn.__version__)
 
 
 @Substitution(
@@ -36,7 +46,7 @@ MAX_INT = np.iinfo(np.int32).max
     n_jobs=_n_jobs_docstring,
     random_state=_random_state_docstring,
 )
-class EasyEnsembleClassifier(BaggingClassifier, _ParamsValidationMixin):
+class EasyEnsembleClassifier(_ParamsValidationMixin, BaggingClassifier):
     """Bag of balanced boosted learners also known as EasyEnsemble.
 
     This algorithm is known as EasyEnsemble [1]_. The classifier is an
@@ -75,14 +85,6 @@ class EasyEnsembleClassifier(BaggingClassifier, _ParamsValidationMixin):
     verbose : int, default=0
         Controls the verbosity of the building process.
 
-    base_estimator : estimator object, default=AdaBoostClassifier()
-        The base AdaBoost classifier used in the inner ensemble. Note that you
-        can set the number of inner learner by passing your own instance.
-
-        .. deprecated:: 0.10
-           `base_estimator` was renamed to `estimator` in version 0.10 and will
-           be removed in 0.12.
-
     Attributes
     ----------
     estimator_ : estimator
@@ -90,15 +92,6 @@ class EasyEnsembleClassifier(BaggingClassifier, _ParamsValidationMixin):
 
         .. versionadded:: 0.10
 
-    base_estimator_ : estimator
-        The base estimator from which the ensemble is grown.
-
-        .. deprecated:: 1.2
-           `base_estimator_` is deprecated in `scikit-learn` 1.2 and will be
-           removed in 1.4. Use `estimator_` instead. When the minimum version
-           of `scikit-learn` supported by `imbalanced-learn` will reach 1.4,
-           this attribute will be removed.
-
     estimators_ : list of estimators
         The collection of fitted base estimators.
 
@@ -181,8 +174,7 @@ class EasyEnsembleClassifier(BaggingClassifier, _ParamsValidationMixin):
     """
 
     # make a deepcopy to not modify the original dictionary
-    if hasattr(BaggingClassifier, "_parameter_constraints"):
-        # scikit-learn >= 1.2
+    if sklearn_version >= parse_version("1.4"):
         _parameter_constraints = copy.deepcopy(BaggingClassifier._parameter_constraints)
     else:
         _parameter_constraints = copy.deepcopy(_bagging_parameter_constraints)
@@ -208,6 +200,9 @@ class EasyEnsembleClassifier(BaggingClassifier, _ParamsValidationMixin):
             "replacement": ["boolean"],
         }
     )
+    # TODO: remove when minimum supported version of scikit-learn is 1.4
+    if "base_estimator" in _parameter_constraints:
+        del _parameter_constraints["base_estimator"]
 
     def __init__(
         self,
@@ -220,18 +215,8 @@ class EasyEnsembleClassifier(BaggingClassifier, _ParamsValidationMixin):
         n_jobs=None,
         random_state=None,
         verbose=0,
-        base_estimator="deprecated",
     ):
-        # TODO: remove when supporting scikit-learn>=1.2
-        bagging_classifier_signature = inspect.signature(super().__init__)
-        estimator_params = {"base_estimator": base_estimator}
-        if "estimator" in bagging_classifier_signature.parameters:
-            estimator_params["estimator"] = estimator
-        else:
-            self.estimator = estimator
-
         super().__init__(
-            **estimator_params,
             n_estimators=n_estimators,
             max_samples=1.0,
             max_features=1.0,
@@ -243,6 +228,7 @@ class EasyEnsembleClassifier(BaggingClassifier, _ParamsValidationMixin):
             random_state=random_state,
             verbose=verbose,
         )
+        self.estimator = estimator
         self.sampling_strategy = sampling_strategy
         self.replacement = replacement
 
@@ -261,46 +247,19 @@ class EasyEnsembleClassifier(BaggingClassifier, _ParamsValidationMixin):
             self._sampling_strategy = self.sampling_strategy
         return y_encoded
 
-    def _validate_estimator(self, default=AdaBoostClassifier()):
+    def _validate_estimator(self, default=AdaBoostClassifier(algorithm="SAMME")):
         """Check the estimator and the n_estimator attribute, set the
         `estimator_` attribute."""
-        if self.estimator is not None and (
-            self.base_estimator not in [None, "deprecated"]
-        ):
-            raise ValueError(
-                "Both `estimator` and `base_estimator` were set. Only set `estimator`."
-            )
-
         if self.estimator is not None:
-            base_estimator = clone(self.estimator)
-        elif self.base_estimator not in [None, "deprecated"]:
-            warnings.warn(
-                "`base_estimator` was renamed to `estimator` in version 0.10 and "
-                "will be removed in 0.12.",
-                FutureWarning,
-            )
-            base_estimator = clone(self.base_estimator)
+            estimator = clone(self.estimator)
         else:
-            base_estimator = clone(default)
+            estimator = clone(default)
 
         sampler = RandomUnderSampler(
             sampling_strategy=self._sampling_strategy,
             replacement=self.replacement,
         )
-        self._estimator = Pipeline(
-            [("sampler", sampler), ("classifier", base_estimator)]
-        )
-        try:
-            self.base_estimator_ = self._estimator
-        except AttributeError:
-            # scikit-learn < 1.2
-            pass
-
-    # TODO: remove when supporting scikit-learn>=1.4
-    @property
-    def estimator_(self):
-        """Estimator used to grow the ensemble."""
-        return self._estimator
+        self.estimator_ = Pipeline([("sampler", sampler), ("classifier", estimator)])
 
     # TODO: remove when supporting scikit-learn>=1.2
     @property
@@ -314,6 +273,7 @@ class EasyEnsembleClassifier(BaggingClassifier, _ParamsValidationMixin):
         )
         return self.n_features_in_
 
+    @_fit_context(prefer_skip_nested_validation=False)
     def fit(self, X, y):
         """Build a Bagging ensemble of estimators from the training set (X, y).
 
@@ -388,3 +348,26 @@ class EasyEnsembleClassifier(BaggingClassifier, _ParamsValidationMixin):
         decisions = sum(all_decisions) / self.n_estimators
 
         return decisions
+
+    @property
+    def base_estimator_(self):
+        """Attribute for older sklearn version compatibility."""
+        error = AttributeError(
+            f"{self.__class__.__name__} object has no attribute 'base_estimator_'."
+        )
+        if sklearn_version < parse_version("1.2"):
+            # The base class require to have the attribute defined. For scikit-learn
+            # > 1.2, we are going to raise an error.
+            try:
+                check_is_fitted(self)
+                return self.estimator_
+            except NotFittedError:
+                raise error
+        raise error
+
+    def _more_tags(self):
+        if self.estimator is None:
+            estimator = AdaBoostClassifier(algorithm="SAMME")
+        else:
+            estimator = self.estimator
+        return {"allow_nan": _safe_tags(estimator, "allow_nan")}
