@@ -9,7 +9,6 @@ from warnings import warn
 
 import numpy as np
 import sklearn
-from joblib import Parallel
 from numpy import float32 as DTYPE
 from numpy import float64 as DOUBLE
 from scipy.sparse import issparse
@@ -24,18 +23,24 @@ from sklearn.ensemble._forest import (
 from sklearn.exceptions import DataConversionWarning
 from sklearn.tree import DecisionTreeClassifier
 from sklearn.utils import _safe_indexing, check_random_state, parse_version
-from sklearn.utils.fixes import delayed
 from sklearn.utils.multiclass import type_of_target
 from sklearn.utils.validation import _check_sample_weight
 
+try:
+    # scikit-learn >= 1.2
+    from sklearn.utils.parallel import Parallel, delayed
+except (ImportError, ModuleNotFoundError):
+    from joblib import Parallel
+    from sklearn.utils.fixes import delayed
+
 from ..base import _ParamsValidationMixin
 from ..pipeline import make_pipeline
 from ..under_sampling import RandomUnderSampler
-from ..under_sampling.base import BaseUnderSampler
 from ..utils import Substitution
 from ..utils._docstring import _n_jobs_docstring, _random_state_docstring
-from ..utils._param_validation import Interval, StrOptions
+from ..utils._param_validation import Hidden, Interval, StrOptions
 from ..utils._validation import check_sampling_strategy
+from ..utils.fixes import _fit_context
 from ._common import _random_forest_classifier_parameter_constraints
 
 MAX_INT = np.iinfo(np.int32).max
@@ -55,6 +60,7 @@ def _local_parallel_build_trees(
     class_weight=None,
     n_samples_bootstrap=None,
     forest=None,
+    missing_values_in_feature_mask=None,
 ):
     # resample before to fit the tree
     X_resampled, y_resampled = sampler.fit_resample(X, y)
@@ -63,46 +69,48 @@ def _local_parallel_build_trees(
     if _get_n_samples_bootstrap is not None:
         n_samples_bootstrap = min(n_samples_bootstrap, X_resampled.shape[0])
 
-    if sklearn_version >= parse_version("1.1"):
-        tree = _parallel_build_trees(
-            tree,
-            bootstrap,
-            X_resampled,
-            y_resampled,
-            sample_weight,
-            tree_idx,
-            n_trees,
-            verbose=verbose,
-            class_weight=class_weight,
-            n_samples_bootstrap=n_samples_bootstrap,
-        )
+    params_parallel_build_trees = {
+        "tree": tree,
+        "X": X_resampled,
+        "y": y_resampled,
+        "sample_weight": sample_weight,
+        "tree_idx": tree_idx,
+        "n_trees": n_trees,
+        "verbose": verbose,
+        "class_weight": class_weight,
+        "n_samples_bootstrap": n_samples_bootstrap,
+    }
+
+    if parse_version(sklearn_version.base_version) >= parse_version("1.4"):
+        # TODO: remove when the minimum supported version of scikit-learn will be 1.4
+        # support for missing values
+        params_parallel_build_trees[
+            "missing_values_in_feature_mask"
+        ] = missing_values_in_feature_mask
+
+    # TODO: remove when the minimum supported version of scikit-learn will be 1.1
+    # change of signature in scikit-learn 1.1
+    if parse_version(sklearn_version.base_version) >= parse_version("1.1"):
+        params_parallel_build_trees["bootstrap"] = bootstrap
     else:
-        # TODO: remove when the minimum version of scikit-learn supported is 1.1
-        tree = _parallel_build_trees(
-            tree,
-            forest,
-            X_resampled,
-            y_resampled,
-            sample_weight,
-            tree_idx,
-            n_trees,
-            verbose=verbose,
-            class_weight=class_weight,
-            n_samples_bootstrap=n_samples_bootstrap,
-        )
+        params_parallel_build_trees["forest"] = forest
+
+    tree = _parallel_build_trees(**params_parallel_build_trees)
+
     return sampler, tree
 
 
 @Substitution(
-    sampling_strategy=BaseUnderSampler._sampling_strategy_docstring,
     n_jobs=_n_jobs_docstring,
     random_state=_random_state_docstring,
 )
-class BalancedRandomForestClassifier(RandomForestClassifier, _ParamsValidationMixin):
+class BalancedRandomForestClassifier(_ParamsValidationMixin, RandomForestClassifier):
     """A balanced random forest classifier.
 
-    A balanced random forest randomly under-samples each boostrap sample to
-    balance it.
+    A balanced random forest differs from a classical random forest by the
+    fact that it will draw a bootstrap sample from the minority class and
+    sample with replacement the same number of samples from the majority
+    class.
 
     Read more in the :ref:`User Guide <forest>`.
 
@@ -183,15 +191,66 @@ class BalancedRandomForestClassifier(RandomForestClassifier, _ParamsValidationMi
     bootstrap : bool, default=True
         Whether bootstrap samples are used when building trees.
 
+        .. versionchanged:: 0.13
+           The default of `bootstrap` will change from `True` to `False` in
+           version 0.13. Bootstrapping is already taken care by the internal
+           sampler using `replacement=True`. This implementation follows the
+           algorithm proposed in [1]_.
+
     oob_score : bool, default=False
         Whether to use out-of-bag samples to estimate
         the generalization accuracy.
 
-    {sampling_strategy}
+    sampling_strategy : float, str, dict, callable, default="auto"
+        Sampling information to sample the data set.
+
+        - When ``float``, it corresponds to the desired ratio of the number of
+          samples in the minority class over the number of samples in the
+          majority class after resampling. Therefore, the ratio is expressed as
+          :math:`\\alpha_{{us}} = N_{{m}} / N_{{rM}}` where :math:`N_{{m}}` is the
+          number of samples in the minority class and
+          :math:`N_{{rM}}` is the number of samples in the majority class
+          after resampling.
+
+          .. warning::
+             ``float`` is only available for **binary** classification. An
+             error is raised for multi-class classification.
+
+        - When ``str``, specify the class targeted by the resampling. The
+          number of samples in the different classes will be equalized.
+          Possible choices are:
+
+            ``'majority'``: resample only the majority class;
+
+            ``'not minority'``: resample all classes but the minority class;
+
+            ``'not majority'``: resample all classes but the majority class;
+
+            ``'all'``: resample all classes;
+
+            ``'auto'``: equivalent to ``'not minority'``.
+
+        - When ``dict``, the keys correspond to the targeted classes. The
+          values correspond to the desired number of samples for each targeted
+          class.
+
+        - When callable, function taking ``y`` and returns a ``dict``. The keys
+          correspond to the targeted classes. The values correspond to the
+          desired number of samples for each class.
+
+        .. versionchanged:: 0.11
+           The default of `sampling_strategy` will change from `"auto"` to
+           `"all"` in version 0.13. This forces to use a bootstrap of the
+           minority class as proposed in [1]_.
 
     replacement : bool, default=False
         Whether or not to sample randomly with replacement or not.
 
+        .. versionchanged:: 0.11
+           The default of `replacement` will change from `False` to `True` in
+           version 0.13. This forces to use a bootstrap of the
+           minority class and draw with replacement as proposed in [1]_.
+
     {n_jobs}
 
     {random_state}
@@ -248,6 +307,25 @@ class BalancedRandomForestClassifier(RandomForestClassifier, _ParamsValidationMi
         .. versionadded:: 0.6
            Added in `scikit-learn` in 0.22
 
+    monotonic_cst : array-like of int of shape (n_features), default=None
+        Indicates the monotonicity constraint to enforce on each feature.
+          - 1: monotonic increase
+          - 0: no constraint
+          - -1: monotonic decrease
+
+        If monotonic_cst is None, no constraints are applied.
+
+        Monotonicity constraints are not supported for:
+          - multiclass classifications (i.e. when `n_classes > 2`),
+          - multioutput classifications (i.e. when `n_outputs_ > 1`),
+          - classifications trained on data with missing values.
+
+        The constraints hold over the probability of the positive class.
+
+        .. versionadded:: 0.12
+           Only supported when scikit-learn >= 1.4 is installed. Otherwise, a
+           `ValueError` is raised.
+
     Attributes
     ----------
     estimator_ : :class:`~sklearn.tree.DecisionTreeClassifier` instance
@@ -256,16 +334,6 @@ class BalancedRandomForestClassifier(RandomForestClassifier, _ParamsValidationMi
 
         .. versionadded:: 0.10
 
-    base_estimator_ : :class:`~sklearn.tree.DecisionTreeClassifier` instance
-        The child estimator template used to create the collection of fitted
-        sub-estimators.
-
-        .. deprecated:: 1.2
-           `base_estimator_` is deprecated in `scikit-learn` 1.2 and will be
-           removed in 1.4. Use `estimator_` instead. When the minimum version
-           of `scikit-learn` supported by `imbalanced-learn` will reach 1.4,
-           this attribute will be removed.
-
     estimators_ : list of :class:`~sklearn.tree.DecisionTreeClassifier`
         The collection of fitted sub-estimators.
 
@@ -345,7 +413,9 @@ class BalancedRandomForestClassifier(RandomForestClassifier, _ParamsValidationMi
     >>> X, y = make_classification(n_samples=1000, n_classes=3,
     ...                            n_informative=4, weights=[0.2, 0.3, 0.5],
     ...                            random_state=0)
-    >>> clf = BalancedRandomForestClassifier(max_depth=2, random_state=0)
+    >>> clf = BalancedRandomForestClassifier(
+    ...     sampling_strategy="all", replacement=True, max_depth=2, random_state=0,
+    ...     bootstrap=False)
     >>> clf.fit(X, y)
     BalancedRandomForestClassifier(...)
     >>> print(clf.feature_importances_)
@@ -356,8 +426,7 @@ class BalancedRandomForestClassifier(RandomForestClassifier, _ParamsValidationMi
     """
 
     # make a deepcopy to not modify the original dictionary
-    if hasattr(RandomForestClassifier, "_parameter_constraints"):
-        # scikit-learn >= 1.2
+    if sklearn_version >= parse_version("1.4"):
         _parameter_constraints = deepcopy(RandomForestClassifier._parameter_constraints)
     else:
         _parameter_constraints = deepcopy(
@@ -366,13 +435,15 @@ class BalancedRandomForestClassifier(RandomForestClassifier, _ParamsValidationMi
 
     _parameter_constraints.update(
         {
+            "bootstrap": ["boolean", Hidden(StrOptions({"warn"}))],
             "sampling_strategy": [
                 Interval(numbers.Real, 0, 1, closed="right"),
                 StrOptions({"auto", "majority", "not minority", "not majority", "all"}),
                 dict,
                 callable,
+                Hidden(StrOptions({"warn"})),
             ],
-            "replacement": ["boolean"],
+            "replacement": ["boolean", Hidden(StrOptions({"warn"}))],
         }
     )
 
@@ -388,10 +459,10 @@ class BalancedRandomForestClassifier(RandomForestClassifier, _ParamsValidationMi
         max_features="sqrt",
         max_leaf_nodes=None,
         min_impurity_decrease=0.0,
-        bootstrap=True,
+        bootstrap="warn",
         oob_score=False,
-        sampling_strategy="auto",
-        replacement=False,
+        sampling_strategy="warn",
+        replacement="warn",
         n_jobs=None,
         random_state=None,
         verbose=0,
@@ -399,27 +470,42 @@ class BalancedRandomForestClassifier(RandomForestClassifier, _ParamsValidationMi
         class_weight=None,
         ccp_alpha=0.0,
         max_samples=None,
+        monotonic_cst=None,
     ):
-        super().__init__(
-            criterion=criterion,
-            max_depth=max_depth,
-            n_estimators=n_estimators,
-            bootstrap=bootstrap,
-            oob_score=oob_score,
-            n_jobs=n_jobs,
-            random_state=random_state,
-            verbose=verbose,
-            warm_start=warm_start,
-            class_weight=class_weight,
-            min_samples_split=min_samples_split,
-            min_samples_leaf=min_samples_leaf,
-            min_weight_fraction_leaf=min_weight_fraction_leaf,
-            max_features=max_features,
-            max_leaf_nodes=max_leaf_nodes,
-            min_impurity_decrease=min_impurity_decrease,
-            ccp_alpha=ccp_alpha,
-            max_samples=max_samples,
-        )
+        params_random_forest = {
+            "criterion": criterion,
+            "max_depth": max_depth,
+            "n_estimators": n_estimators,
+            "bootstrap": bootstrap,
+            "oob_score": oob_score,
+            "n_jobs": n_jobs,
+            "random_state": random_state,
+            "verbose": verbose,
+            "warm_start": warm_start,
+            "class_weight": class_weight,
+            "min_samples_split": min_samples_split,
+            "min_samples_leaf": min_samples_leaf,
+            "min_weight_fraction_leaf": min_weight_fraction_leaf,
+            "max_features": max_features,
+            "max_leaf_nodes": max_leaf_nodes,
+            "min_impurity_decrease": min_impurity_decrease,
+            "ccp_alpha": ccp_alpha,
+            "max_samples": max_samples,
+        }
+        # TODO: remove when the minimum supported version of scikit-learn will be 1.4
+        if parse_version(sklearn_version.base_version) >= parse_version("1.4"):
+            # use scikit-learn support for monotonic constraints
+            params_random_forest["monotonic_cst"] = monotonic_cst
+        else:
+            if monotonic_cst is not None:
+                raise ValueError(
+                    "Monotonic constraints are not supported for scikit-learn "
+                    "version < 1.4."
+                )
+            # create an attribute for compatibility with other scikit-learn tools such
+            # as HTML representation.
+            self.monotonic_cst = monotonic_cst
+        super().__init__(**params_random_forest)
 
         self.sampling_strategy = sampling_strategy
         self.replacement = replacement
@@ -433,19 +519,13 @@ class BalancedRandomForestClassifier(RandomForestClassifier, _ParamsValidationMi
             base_estimator = self.base_estimator
 
         if base_estimator is not None:
-            self._estimator = clone(base_estimator)
+            self.estimator_ = clone(base_estimator)
         else:
-            self._estimator = clone(default)
-
-        try:
-            # scikit-learn < 1.2
-            self.base_estimator_ = self._estimator
-        except AttributeError:
-            pass
+            self.estimator_ = clone(default)
 
         self.base_sampler_ = RandomUnderSampler(
             sampling_strategy=self._sampling_strategy,
-            replacement=self.replacement,
+            replacement=self._replacement,
         )
 
     def _make_sampler_estimator(self, random_state=None):
@@ -453,7 +533,7 @@ class BalancedRandomForestClassifier(RandomForestClassifier, _ParamsValidationMi
         Warning: This method should be used to properly instantiate new
         sub-estimators.
         """
-        estimator = clone(self._estimator)
+        estimator = clone(self.estimator_)
         estimator.set_params(**{p: getattr(self, p) for p in self.estimator_params})
         sampler = clone(self.base_sampler_)
 
@@ -463,6 +543,7 @@ class BalancedRandomForestClassifier(RandomForestClassifier, _ParamsValidationMi
 
         return estimator, sampler
 
+    @_fit_context(prefer_skip_nested_validation=True)
     def fit(self, X, y, sample_weight=None):
         """Build a forest of trees from the training set (X, y).
 
@@ -490,14 +571,81 @@ class BalancedRandomForestClassifier(RandomForestClassifier, _ParamsValidationMi
             The fitted instance.
         """
         self._validate_params()
+        # TODO: remove in 0.13
+        if self.sampling_strategy == "warn":
+            warn(
+                "The default of `sampling_strategy` will change from `'auto'` to "
+                "`'all'` in version 0.13. This change will follow the implementation "
+                "proposed in the original paper. Set to `'all'` to silence this "
+                "warning and adopt the future behaviour.",
+                FutureWarning,
+            )
+            self._sampling_strategy = "auto"
+        else:
+            self._sampling_strategy = self.sampling_strategy
+
+        if self.replacement == "warn":
+            warn(
+                "The default of `replacement` will change from `False` to "
+                "`True` in version 0.13. This change will follow the implementation "
+                "proposed in the original paper. Set to `True` to silence this "
+                "warning and adopt the future behaviour.",
+                FutureWarning,
+            )
+            self._replacement = False
+        else:
+            self._replacement = self.replacement
+
+        if self.bootstrap == "warn":
+            warn(
+                "The default of `bootstrap` will change from `True` to "
+                "`False` in version 0.13. This change will follow the implementation "
+                "proposed in the original paper. Set to `False` to silence this "
+                "warning and adopt the future behaviour.",
+                FutureWarning,
+            )
+            self._bootstrap = True
+        else:
+            self._bootstrap = self.bootstrap
+
         # Validate or convert input data
         if issparse(y):
             raise ValueError("sparse multilabel-indicator for y is not supported.")
+
+        # TODO: remove when the minimum supported version of scipy will be 1.4
+        # Support for missing values
+        if parse_version(sklearn_version.base_version) >= parse_version("1.4"):
+            force_all_finite = False
+        else:
+            force_all_finite = True
+
         X, y = self._validate_data(
-            X, y, multi_output=True, accept_sparse="csc", dtype=DTYPE
+            X,
+            y,
+            multi_output=True,
+            accept_sparse="csc",
+            dtype=DTYPE,
+            force_all_finite=force_all_finite,
         )
+
+        # TODO: remove when the minimum supported version of scikit-learn will be 1.4
+        if parse_version(sklearn_version.base_version) >= parse_version("1.4"):
+            # _compute_missing_values_in_feature_mask checks if X has missing values and
+            # will raise an error if the underlying tree base estimator can't handle
+            # missing values. Only the criterion is required to determine if the tree
+            # supports missing values.
+            estimator = type(self.estimator)(criterion=self.criterion)
+            missing_values_in_feature_mask = (
+                estimator._compute_missing_values_in_feature_mask(
+                    X, estimator_name=self.__class__.__name__
+                )
+            )
+        else:
+            missing_values_in_feature_mask = None
+
         if sample_weight is not None:
             sample_weight = _check_sample_weight(sample_weight, X)
+
         self._n_features = X.shape[1]
 
         if issparse(X):
@@ -527,7 +675,7 @@ class BalancedRandomForestClassifier(RandomForestClassifier, _ParamsValidationMi
         if getattr(y, "dtype", None) != DOUBLE or not y.flags.contiguous:
             y_encoded = np.ascontiguousarray(y_encoded, dtype=DOUBLE)
 
-        if isinstance(self.sampling_strategy, dict):
+        if isinstance(self._sampling_strategy, dict):
             self._sampling_strategy = {
                 np.where(self.classes_[0] == key)[0][0]: value
                 for key, value in check_sampling_strategy(
@@ -537,7 +685,7 @@ class BalancedRandomForestClassifier(RandomForestClassifier, _ParamsValidationMi
                 ).items()
             }
         else:
-            self._sampling_strategy = self.sampling_strategy
+            self._sampling_strategy = self._sampling_strategy
 
         if expanded_class_weight is not None:
             if sample_weight is not None:
@@ -553,7 +701,7 @@ class BalancedRandomForestClassifier(RandomForestClassifier, _ParamsValidationMi
         # Check parameters
         self._validate_estimator()
 
-        if not self.bootstrap and self.oob_score:
+        if not self._bootstrap and self.oob_score:
             raise ValueError("Out of bag estimation only available if bootstrap=True")
 
         random_state = check_random_state(self.random_state)
@@ -605,7 +753,7 @@ class BalancedRandomForestClassifier(RandomForestClassifier, _ParamsValidationMi
                 delayed(_local_parallel_build_trees)(
                     s,
                     t,
-                    self.bootstrap,
+                    self._bootstrap,
                     X,
                     y_encoded,
                     sample_weight,
@@ -615,6 +763,7 @@ class BalancedRandomForestClassifier(RandomForestClassifier, _ParamsValidationMi
                     class_weight=self.class_weight,
                     n_samples_bootstrap=n_samples_bootstrap,
                     forest=self,
+                    missing_values_in_feature_mask=missing_values_in_feature_mask,
                 )
                 for i, (s, t) in enumerate(zip(samplers, trees))
             )
@@ -745,12 +894,6 @@ class BalancedRandomForestClassifier(RandomForestClassifier, _ParamsValidationMi
 
         return oob_pred
 
-    # TODO: remove when supporting scikit-learn>=1.4
-    @property
-    def estimator_(self):
-        """Estimator used to grow the ensemble."""
-        return self._estimator
-
     # TODO: remove when supporting scikit-learn>=1.2
     @property
     def n_features_(self):
