@@ -154,7 +154,6 @@ def test_tabs():
     for importer, modname, ispkg in walk_packages(
         imblearn.__path__, prefix="imblearn."
     ):
-
         if IS_PYPY:
             continue
 
@@ -206,14 +205,18 @@ def test_fit_docstring_attributes(name, Estimator):
     X = _enforce_estimator_tags_x(est, X)
 
     if "oob_score" in est.get_params():
-        est.set_params(oob_score=True)
+        est.set_params(bootstrap=True, oob_score=True)
 
     if is_sampler(est):
         est.fit_resample(X, y)
     else:
         est.fit(X, y)
 
-    skipped_attributes = set([])
+    skipped_attributes = set(
+        [
+            "base_estimator_",  # this attribute exist with old version of sklearn
+        ]
+    )
 
     for attr in attributes:
         if attr.name in skipped_attributes:
