@@ -246,7 +246,7 @@ def test_pipeline_init():
 
     # Test clone
     pipe2 = clone(pipe)
-    assert not pipe.named_steps["svc"] is pipe2.named_steps["svc"]
+    assert pipe.named_steps["svc"] is not pipe2.named_steps["svc"]
 
     # Check that apart from estimators, the parameters are the same
     params = pipe.get_params(deep=True)
@@ -485,7 +485,7 @@ def test_set_pipeline_steps():
     pipeline.set_params(steps=[("junk", ())])
     with raises(TypeError):
         pipeline.fit([[1]], [1])
-    with raises(TypeError):
+    with raises(AttributeError):
         pipeline.fit_transform([[1]], [1])
 
 
@@ -1163,7 +1163,6 @@ def test_predict_with_predict_params():
 
 
 def test_resampler_last_stage_passthrough():
-
     X, y = make_classification(
         n_classes=2,
         class_sep=2,
